@@ -42,11 +42,11 @@
 #include <libgnomeui/gnome-stock-icons.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
-/* The width of the progress bar determines the minimum width of the
- * window. It will be wider only if the font is really huge and the
- * fixed labels don't fit in the window otherwise.
+/* The default width of the progress dialog. It will be wider
+ * only if the font is really huge and the fixed labels don't
+ * fit in the window otherwise.
  */
-#define PROGRESS_BAR_WIDTH 350
+#define PROGRESS_DIALOG_WIDTH 400
 
 #define OUTER_BORDER       5
 #define HORIZONTAL_SPACING 3
@@ -97,10 +97,10 @@ nautilus_file_operations_progress_update (NautilusFileOperationsProgress *progre
 		return;
 	}
 
-	gtk_progress_configure (GTK_PROGRESS (progress->details->progress_bar),
-				progress->details->bytes_copied,
-				0.0,
-				progress->details->bytes_total);
+	gtk_progress_bar_set_fraction
+		(GTK_PROGRESS_BAR (progress->details->progress_bar),
+		 (double) progress->details->bytes_copied /
+		 progress->details->bytes_total);
 }
 
 static void
@@ -126,6 +126,19 @@ close_callback (GtkDialog *dialog)
 {
 }
 
+/* GObject methods. */
+static void
+nautilus_file_operations_progress_finalize (GObject *object)
+{
+	NautilusFileOperationsProgress *progress;
+
+	progress = NAUTILUS_FILE_OPERATIONS_PROGRESS (object);
+
+	g_free (progress->details);
+
+	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+}
+
 /* GtkObject methods.  */
 
 static void
@@ -137,10 +150,9 @@ nautilus_file_operations_progress_destroy (GtkObject *object)
 
 	if (progress->details->delayed_close_timeout_id != 0) {
 		g_source_remove (progress->details->delayed_close_timeout_id);
+		progress->details->delayed_close_timeout_id = 0;
 	}
-
-	g_free (progress->details);
-
+	
 	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
@@ -228,11 +240,7 @@ nautilus_file_operations_progress_init (NautilusFileOperationsProgress *progress
 
 	/* progress bar */
 	progress->details->progress_bar = gtk_progress_bar_new ();
-	gtk_progress_bar_set_bar_style (GTK_PROGRESS_BAR (progress->details->progress_bar),
-					GTK_PROGRESS_CONTINUOUS);
-	gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (progress->details->progress_bar),
-					  GTK_PROGRESS_LEFT_TO_RIGHT);
-	gtk_widget_set_usize (progress->details->progress_bar, PROGRESS_BAR_WIDTH, -1);
+	gtk_window_set_default_size (GTK_WINDOW (progress), PROGRESS_DIALOG_WIDTH, -1);
 	gtk_box_pack_start (vbox, progress->details->progress_bar, FALSE, TRUE, 0);
 	gtk_widget_show (progress->details->progress_bar);
 
@@ -257,14 +265,19 @@ nautilus_file_operations_progress_init (NautilusFileOperationsProgress *progress
 static void
 nautilus_file_operations_progress_class_init (NautilusFileOperationsProgressClass *klass)
 {
+	GObjectClass *gobject_class;
 	GtkObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 	GtkDialogClass *dialog_class;
 
+	gobject_class = G_OBJECT_CLASS (klass);
 	object_class = GTK_OBJECT_CLASS (klass);
 	widget_class = GTK_WIDGET_CLASS (klass);
 	dialog_class = GTK_DIALOG_CLASS (klass);
 
+
+	gobject_class->finalize = nautilus_file_operations_progress_finalize;
+	
 	object_class->destroy = nautilus_file_operations_progress_destroy;
 
 	/* The progress dialog should not have a title and a close box.
