@@ -30,18 +30,19 @@
 #include <config.h>
 #include "nautilus-view-standard-main.h"
 
+#include <X11/Xlib.h>
 #include <bonobo/bonobo-generic-factory.h>
 #include <bonobo/bonobo-main.h>
+#include <bonobo/bonobo-ui-main.h>
+#include <gdk/gdkx.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
-#include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-init.h>
+#include <libgnomeui/gnome-client.h>
+#include <libgnomeui/gnome-ui-init.h>
 #include <libgnomevfs/gnome-vfs-init.h>
-#include <liboaf/liboaf.h>
-#include <gdk/gdkx.h>
-#include <X11/Xlib.h>
 #include <stdlib.h>
+#include <string.h>
 
 #define N_IDLE_SECONDS_BEFORE_QUIT  5
 
@@ -61,20 +62,24 @@ delayed_quit_timeout_callback (gpointer data)
 	callback_data = (CallbackData *) data;
 	callback_data->delayed_quit_timeout_id = 0;
 	gtk_main_quit ();
+
 	return FALSE;
 }
 
 static void
-object_destroyed (GtkObject     *object,
-		  CallbackData  *callback_data)
+object_destroyed (GObject      *object,
+		  CallbackData *callback_data)
 {
-	g_assert (GTK_IS_OBJECT (object));
+	g_assert (G_IS_OBJECT (object));
 
 	callback_data->object_count--;
-	if (callback_data->object_count <= 0 && callback_data->delayed_quit_timeout_id == 0) {
-		callback_data->delayed_quit_timeout_id = g_timeout_add (N_IDLE_SECONDS_BEFORE_QUIT * 1000,
-		                                                        delayed_quit_timeout_callback,
-		                                                        callback_data);
+
+	if (callback_data->object_count <= 0 &&
+	    callback_data->delayed_quit_timeout_id == 0) {
+		callback_data->delayed_quit_timeout_id = 
+			g_timeout_add (N_IDLE_SECONDS_BEFORE_QUIT * 1000,
+				       delayed_quit_timeout_callback,
+				       callback_data);
 	}
 }
 
@@ -111,8 +116,9 @@ make_object (BonoboGenericFactory *factory,
 		g_source_remove (callback_data->delayed_quit_timeout_id);
 		callback_data->delayed_quit_timeout_id = 0;
 	}
-	gtk_signal_connect (GTK_OBJECT (view), "destroy",
-			    object_destroyed, callback_data);
+	g_signal_connect (view, "destroy",
+			  G_CALLBACK (object_destroyed),
+			  callback_data);
 
 	return BONOBO_OBJECT (view);
 }
@@ -157,7 +163,6 @@ nautilus_view_standard_main_multi (const char *executable_name,
 				   GVoidFunc post_initialize_callback,
 				   void *user_data)
 {
-	CORBA_ORB orb;
 	BonoboGenericFactory *factory;
 	CallbackData callback_data;
 	char *registration_id;
@@ -179,26 +184,24 @@ nautilus_view_standard_main_multi (const char *executable_name,
 	}
 
 	/* Initialize gettext support if needed  */
-#ifdef ENABLE_NLS
 	if (gettext_package_name != NULL
 	    && gettext_locale_directory != NULL) {
 		bindtextdomain (gettext_package_name, gettext_locale_directory);
+		bind_textdomain_codeset (gettext_package_name, "UTF-8");
 		textdomain (gettext_package_name);
 	}
-#endif
-
-	/* Disable session manager connection */
-	gnome_client_disable_master_connection ();
-
-	gnomelib_register_popt_table (oaf_popt_options, oaf_get_popt_table_name ());
-	orb = oaf_init (argc, argv);
 
 	/* Initialize libraries. */
-        gnome_init (executable_name, version, argc, argv); 
-	gdk_rgb_init ();
-	g_thread_init (NULL);
-	gnome_vfs_init ();
-	bonobo_init (orb, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL);
+	gnome_program_init (executable_name, version,
+			    LIBGNOMEUI_MODULE,
+			    argc, argv,
+			    NULL);
+	
+	bonobo_ui_init (executable_name, version, &argc, argv);
+
+	/* Disable session manager connection */
+	g_object_set (G_OBJECT (gnome_program_get()),
+	              GNOME_CLIENT_PARAM_SM_CONNECT, FALSE, NULL);
 
 	if (post_initialize_callback != NULL) {
 		(* post_initialize_callback) ();
@@ -212,20 +215,20 @@ nautilus_view_standard_main_multi (const char *executable_name,
 	callback_data.delayed_quit_timeout_id = 0;
 
 	/* Create the factory. */
-        registration_id = oaf_make_registration_id (factory_iid, 
-						    DisplayString (GDK_DISPLAY ()));
-	factory = bonobo_generic_factory_new_multi (registration_id, 
-						    make_object,
-						    &callback_data);
+        registration_id = bonobo_activation_make_registration_id (factory_iid, DisplayString (gdk_display));
+	factory = bonobo_generic_factory_new (registration_id, 
+					      make_object,
+					      &callback_data);
 	g_free (registration_id);
 
-	/* Loop until we have no more objects. */
-	do {
-		bonobo_main ();
-	} while (callback_data.object_count > 0 || callback_data.delayed_quit_timeout_id != 0);
-
-	/* Let the factory go. */
-	bonobo_object_unref (BONOBO_OBJECT (factory));
+	if (factory != NULL) {
+		/* Loop until we have no more objects. */
+		bonobo_activate ();
+		do {
+			gtk_main ();
+		} while (callback_data.object_count > 0 || callback_data.delayed_quit_timeout_id != 0);
+		bonobo_object_unref (factory);
+	}
 
 	gnome_vfs_shutdown ();
 
@@ -313,6 +316,5 @@ typedef GtkType (* TypeFunc) (void);
 NautilusView *
 nautilus_view_create_from_get_type_function (const char *iid, void *user_data)
 {
-	return NAUTILUS_VIEW (gtk_object_new (((TypeFunc) (user_data)) (), NULL));
+	return NAUTILUS_VIEW (g_object_new (((TypeFunc) (user_data)) (), NULL));
 }
-
