@@ -122,7 +122,7 @@ create_object (PortableServer_Servant servant,
 	} else if (strcmp (iid, METAFILE_FACTORY_IID) == 0) {
 		object = BONOBO_OBJECT (nautilus_metafile_factory_get_instance ());
 	} else {
-		return CORBA_OBJECT_NIL;
+		object = CORBA_OBJECT_NIL;
 	}
 
 	return CORBA_Object_duplicate (BONOBO_OBJREF (object), ev);
@@ -141,16 +141,12 @@ nautilus_application_instance_init (NautilusApplication *application)
 	application->undo_manager = nautilus_undo_manager_new ();
 
 	/* Watch for volume mounts so we can restore open windows */
-	g_signal_connect (nautilus_volume_monitor_get (),
-			    "volume_mounted",
-			    G_CALLBACK (volume_mounted_callback),
-			    application);
+	g_signal_connect (nautilus_volume_monitor_get (), "volume_mounted",
+			  G_CALLBACK (volume_mounted_callback), application);
 
 	/* Watch for volume unmounts so we can close open windows */
-	g_signal_connect (nautilus_volume_monitor_get (),
-			    "volume_unmounted",
-			    G_CALLBACK (volume_unmounted_callback),
-			    application);
+	g_signal_connect (nautilus_volume_monitor_get (), "volume_unmounted",
+			  G_CALLBACK (volume_unmounted_callback), application);
 }
 
 NautilusApplication *
@@ -176,7 +172,7 @@ nautilus_application_destroy (BonoboObject *object)
 
 	nautilus_bookmarks_exiting ();
 	
-	bonobo_object_unref (BONOBO_OBJECT (application->undo_manager));
+	bonobo_object_unref (application->undo_manager);
 
 	EEL_CALL_PARENT (BONOBO_OBJECT_CLASS, destroy, (object));
 }
@@ -417,8 +413,8 @@ nautilus_application_startup (NautilusApplication *application,
 	/* Start up the factory. */
 	while (TRUE) {
 		/* Try to register the file manager view factory. */
-		result = bonobo_activation_active_server_register (
-			FACTORY_IID, BONOBO_OBJREF (application));
+		result = bonobo_activation_active_server_register
+			(FACTORY_IID, BONOBO_OBJREF (application));
 
 		switch (result) {
 		case Bonobo_ACTIVATION_REG_SUCCESS:
@@ -642,12 +638,12 @@ nautilus_application_create_window (NautilusApplication *application)
 						  "app_id", "nautilus", NULL));
 	
 	g_signal_connect (window, 
-			    "delete_event", G_CALLBACK (nautilus_window_delete_event_callback),
-                    	    NULL);
+			  "delete_event", G_CALLBACK (nautilus_window_delete_event_callback),
+			  NULL);
 
 	g_signal_connect (window,
-			    "destroy", G_CALLBACK (nautilus_application_destroyed_window),
-			    application);
+			  "destroy", G_CALLBACK (nautilus_application_destroyed_window),
+			  application);
 
 	nautilus_application_window_list = g_list_prepend (nautilus_application_window_list, window);
 
@@ -1017,9 +1013,8 @@ is_kdesktop_present (void)
 }
 
 static void
-nautilus_application_class_init (NautilusApplicationClass *klass)
+nautilus_application_class_init (NautilusApplicationClass *class)
 {
-	BONOBO_OBJECT_CLASS (klass)->destroy = nautilus_application_destroy;
-	
-	BONOBO_GENERIC_FACTORY_CLASS (klass)->epv.createObject = create_object;
+	BONOBO_OBJECT_CLASS (class)->destroy = nautilus_application_destroy;
+	BONOBO_GENERIC_FACTORY_CLASS (class)->epv.createObject = create_object;
 }
