@@ -537,9 +537,9 @@ static void
 cancel_viewed_file_changed_callback (NautilusWindow *window)
 {
         if (window->details->viewed_file != NULL) {
-                gtk_signal_disconnect_by_func (GTK_OBJECT (window->details->viewed_file),
-                                               G_CALLBACK (viewed_file_changed_callback),
-                                               window);
+                g_signal_handlers_disconnect_by_func (window->details->viewed_file,
+                                                      G_CALLBACK (viewed_file_changed_callback),
+                                                      window);
         }
 }
 
@@ -861,14 +861,21 @@ load_new_location_in_sidebar_panels (NautilusWindow *window,
                                      GList *selection,
                                      NautilusViewFrame *view_to_skip)
 {
-	GList *sidebar_panels;
+        GList *l;
         GList *node;
+	GList *sidebar_panels;
         NautilusViewFrame *view;
 
 	/* Copy the list before traversing it, because during a failure in
 	 * load_new..., list could be modified and bad things would happen
+         * also reference each object in case of re-enterency eg. window close.
 	 */
-	sidebar_panels = g_list_copy (window->sidebar_panels);
+        sidebar_panels = NULL;
+        for (l = window->sidebar_panels; l; l = l->next) {
+                sidebar_panels = g_list_prepend (sidebar_panels,
+                                                 g_object_ref (l->data));
+        }
+
         for (node = sidebar_panels; node != NULL; node = node->next) {
                 view = node->data;
                 if (view != view_to_skip
@@ -876,6 +883,11 @@ load_new_location_in_sidebar_panels (NautilusWindow *window,
                         load_new_location_in_one_view (view, location, selection);
                 }
         }
+
+        for (l = sidebar_panels; l; l = l->next) {
+                g_object_unref (l->data);
+        }
+
 	g_list_free (sidebar_panels);
 }
 
@@ -1026,11 +1038,11 @@ report_sidebar_panel_failure_to_user (NautilusWindow *window, NautilusViewFrame
 static void
 disconnect_and_destroy_sidebar_panel (NautilusWindow *window, NautilusViewFrame *view)
 {
-        gtk_widget_ref (GTK_WIDGET (view));
+        g_object_ref (view);
 	disconnect_view (window, view);
         nautilus_window_remove_sidebar_panel (window, view);
-	gtk_widget_destroy (GTK_WIDGET (view));
-        gtk_widget_unref (GTK_WIDGET (view));
+	gtk_object_destroy (GTK_OBJECT (view));
+        g_object_unref (view);
 }
 
 static void
@@ -1314,7 +1326,18 @@ determined_initial_view_callback (NautilusDetermineViewHandle *handle,
                                    "don't have an index, that the Medusa indexer is running."));
 			dialog_title = g_strdup (_("Searching Unavailable"));
 			break;
-		} /* else fall through */
+		} else {
+			/* This is a special case for the smb: module */
+			if (eel_istr_has_prefix(location, "smb:") == 0)
+			{
+				error_message = g_strdup_printf
+					(_("Couldn't display \"%s\", because Nautilus cannot contact the SMB master browser.\n"
+					"Check that an SMB server is running in the local network."),
+					uri_for_display);
+				break;
+			}
+		}
+		/* else fall through */
         default:
                 error_message = g_strdup_printf (_("Nautilus cannot display \"%s\"."),
                                                  uri_for_display);
@@ -1336,18 +1359,20 @@ determined_initial_view_callback (NautilusDetermineViewHandle *handle,
 			/* the user could have typed in a home directory that doesn't exist,
 			   in which case going home would cause an infinite loop, so we
 			   better test for that */
-			
-			home_uri = eel_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
-			if (!eel_uris_match (home_uri, location)) {	
-				nautilus_window_go_home (NAUTILUS_WINDOW (window));
-			} else {
+
+                        if (!eel_uris_match (location, "file:///")) {
+                                home_uri = eel_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
+                                if (!eel_uris_match (home_uri, location)) {	
+                                        nautilus_window_go_home (NAUTILUS_WINDOW (window));
+                                } else {
 				/* the last fallback is to go to a known place that can't be deleted! */
-				nautilus_window_go_to (NAUTILUS_WINDOW (window), "file:///");
-			}
-			g_free (home_uri);
+                                        nautilus_window_go_to (NAUTILUS_WINDOW (window), "file:///");
+                                }
+                                g_free (home_uri);
+                        }
 		} else {
-                /* Since this is a window, destroying it will also unref it. */
-                gtk_object_destroy (GTK_OBJECT (window));
+                        /* Since this is a window, destroying it will also unref it. */
+                        gtk_object_destroy (GTK_OBJECT (window));
        		}
         } else {
                 /* Clean up state of already-showing window */
@@ -1392,6 +1417,8 @@ begin_location_change (NautilusWindow *window,
                   || type == NAUTILUS_LOCATION_CHANGE_FORWARD
                   || distance == 0);
 
+        g_object_ref (window);
+
         end_location_change (window);
         
         nautilus_window_allow_stop (window, TRUE);
@@ -1424,9 +1451,9 @@ begin_location_change (NautilusWindow *window,
         nautilus_directory_unref (directory);
         
         window->details->determine_view_handle = nautilus_determine_initial_view
-                (location,
-                 determined_initial_view_callback,
-                 window);
+                (location, determined_initial_view_callback, window);
+
+        g_object_unref (window);
 }
 
 static void
@@ -1964,12 +1991,8 @@ view_loaded_callback (NautilusViewFrame *view,
 static void
 connect_view (NautilusWindow *window, NautilusViewFrame *view)
 {
-	GtkObject *view_object;
-	
-	view_object = GTK_OBJECT (view);
-
-	#define CONNECT(signal) gtk_signal_connect \
-        	(view_object, #signal, \
+	#define CONNECT(signal) g_signal_connect \
+        	(view, #signal, \
                  G_CALLBACK (signal##_callback), window);
         FOR_EACH_NAUTILUS_WINDOW_SIGNAL (CONNECT)
 	#undef CONNECT
@@ -1978,8 +2001,6 @@ connect_view (NautilusWindow *window, NautilusViewFrame *view)
 static void
 disconnect_view (NautilusWindow *window, NautilusViewFrame *view)
 {
-	GtkObject *view_object;
-	
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
 	if (view == NULL) {
@@ -1988,10 +2009,8 @@ disconnect_view (NautilusWindow *window, NautilusViewFrame *view)
 
 	g_assert (NAUTILUS_IS_VIEW_FRAME (view));
 
-	view_object = GTK_OBJECT (view);
-
-#define DISCONNECT(signal) gtk_signal_disconnect_by_func \
-        	(view_object, \
+#define DISCONNECT(signal) g_signal_handlers_disconnect_by_func \
+        	(view, \
         	 G_CALLBACK (signal##_callback), window);
         FOR_EACH_NAUTILUS_WINDOW_SIGNAL (DISCONNECT)
 #undef DISCONNECT
@@ -2007,15 +2026,17 @@ disconnect_view_callback (gpointer list_item_data, gpointer callback_data)
 void
 nautilus_window_manage_views_destroy (NautilusWindow *window)
 {
-        free_location_change (window);
-
 	/* Disconnect view signals here so they don't trigger when
 	 * views are destroyed.
          */
 	g_list_foreach (window->sidebar_panels, disconnect_view_callback, window);
         disconnect_view (window, window->content_view);
+}
 
-        /* Cancel callbacks. */
+void
+nautilus_window_manage_views_finalize (NautilusWindow *window)
+{
+        free_location_change (window);
         cancel_viewed_file_changed_callback (window);
 }
 
