@@ -28,7 +28,6 @@
 /* nautilus-window.c: Implementation of the main window object */
 
 #include <config.h>
-#include <unistd.h>
 #include "nautilus-window-private.h"
 
 #include "nautilus-application.h"
@@ -59,6 +58,7 @@
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkvbox.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-macros.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-messagebox.h>
 #include <libgnomeui/gnome-uidefs.h>
@@ -81,6 +81,7 @@
 #include <libnautilus/nautilus-undo.h>
 #include <math.h>
 #include <sys/time.h>
+#include <unistd.h>
 
 /* FIXME bugzilla.gnome.org 41243: 
  * We should use inheritance instead of these special cases
@@ -112,37 +113,17 @@ enum {
 	ARG_APP
 };
 
-static GList *history_list = NULL;
+static GList *history_list;
 
 static GdkPixmap *mini_icon_pixmap;
 static GdkBitmap *mini_icon_mask;
 
-static void nautilus_window_class_init          (NautilusWindowClass *klass);
-static void nautilus_window_init                (NautilusWindow      *window);
-static void nautilus_window_destroy                   (GtkObject           *object);
-static void nautilus_window_set_property              (GObject             *object,
-						       guint                arg_id,
-						       const GValue        *value,
-						       GParamSpec          *pspec);
+static void update_sidebar_panels_from_preferences (NautilusWindow *window);
+static void sidebar_panels_changed_callback        (gpointer        user_data);
+static void cancel_view_as_callback                (NautilusWindow *window);
 
-static void nautilus_window_get_property              (GObject           *object,
-						       guint              arg_id,
-						       GValue            *value,
-						       GParamSpec        *pspec);
-
-static void nautilus_window_size_request              (GtkWidget           *widget,
-						       GtkRequisition      *requisition);
-static void nautilus_window_realize                   (GtkWidget           *widget);
-static void update_sidebar_panels_from_preferences    (NautilusWindow      *window);
-static void sidebar_panels_changed_callback           (gpointer             user_data);
-static void nautilus_window_show                      (GtkWidget           *widget);
-static void nautilus_window_unrealize                 (GtkWidget           *widget);
-static void cancel_view_as_callback                   (NautilusWindow      *window);
-static void real_add_current_location_to_history_list (NautilusWindow      *window);
-
-EEL_CLASS_BOILERPLATE (NautilusWindow,
-			      nautilus_window,
-			      BONOBO_TYPE_WINDOW)
+GNOME_CLASS_BOILERPLATE (NautilusWindow, nautilus_window,
+			 BonoboWindow, BONOBO_TYPE_WINDOW)
 
 static void
 unref_mini_icon (void)
@@ -154,82 +135,52 @@ unref_mini_icon (void)
 }
 
 static void
-nautilus_window_class_init (NautilusWindowClass *klass)
+load_mini_icon (void)
 {
-	GObjectClass   *gobject_class;
-	GtkObjectClass *object_class;
-	GtkWidgetClass *widget_class;
 	char *filename;
 	GdkPixbuf *pixbuf;
-	GList *icons;
+       	static gboolean loaded;
+
+	if (loaded) {
+		return;
+	}
+	loaded = TRUE;
+
+	filename = nautilus_pixmap_file ("nautilus-mini-logo.png");
+	if (filename == NULL) {
+		return;
+	}
+
+	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+	if (pixbuf != NULL) {
+		gdk_pixbuf_render_pixmap_and_mask
+			(pixbuf, &mini_icon_pixmap, &mini_icon_mask,
+			 EEL_STANDARD_ALPHA_THRESHHOLD);
+		g_object_unref (pixbuf);
+		eel_debug_call_at_shutdown (unref_mini_icon);
+	}
+	g_free (filename);
+}
+
+static void
+set_up_default_icon_list (void)
+{
+	GList *icon_list;
 	guint i;
-	char *icon_filenames[] = { "nautilus-mini-logo.png", "nautilus-launch-icon.png" };
-	
-	gobject_class = (GObjectClass *) klass;
-	object_class = (GtkObjectClass *) klass;
-	widget_class = (GtkWidgetClass *) klass;
+	GdkPixbuf *pixbuf;
+	const char *icon_filenames[] = { "nautilus-mini-logo.png", "nautilus-launch-icon.png" };
 
-	object_class->destroy = nautilus_window_destroy;
-	gobject_class->get_property = nautilus_window_get_property;
-	gobject_class->set_property = nautilus_window_set_property;
-	
-	widget_class->show = nautilus_window_show;
-	widget_class->unrealize = nautilus_window_unrealize;
-	
-	g_object_class_install_property (gobject_class,
-					 ARG_APP_ID,
-					 g_param_spec_string ("app_id",
-							      _("Application ID"),
-							      _("The application ID of the window."),
-							      NULL,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-
-	g_object_class_install_property (gobject_class,
-					 ARG_APP,
-					 g_param_spec_object ("app",
-							      _("Application"),
-							      _("The NautilusApplication associated with this window."),
-							      NAUTILUS_TYPE_APPLICATION,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-	
-	widget_class->realize = nautilus_window_realize;
-	widget_class->size_request = nautilus_window_size_request;
-
-	klass->add_current_location_to_history_list
-		= real_add_current_location_to_history_list;
-
-	/* Load the mini icon for backwards compat */
-        filename = nautilus_pixmap_file ("nautilus-mini-logo.png");
-        if (filename != NULL) {
-                pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
-                if (pixbuf != NULL) {
-                        gdk_pixbuf_render_pixmap_and_mask
-				(pixbuf, &mini_icon_pixmap, &mini_icon_mask,
-				 EEL_STANDARD_ALPHA_THRESHHOLD);
-			g_object_unref (pixbuf);
-			eel_debug_call_at_shutdown (unref_mini_icon);
-		}
-        	g_free (filename);
-	}
-
-
-	/* Set default icon list for all windows */
-	icons = NULL;
+	icon_list = NULL;
 	for (i = 0; i < G_N_ELEMENTS (icon_filenames); i++) {
-		filename = nautilus_pixmap_file (icon_filenames[i]);
-		if (filename != NULL) {
-			pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
-			if (pixbuf != NULL) {
-				icons = g_list_prepend (icons, pixbuf);
-			}
-			g_free (filename);
+		pixbuf = gdk_pixbuf_new_from_file (icon_filenames[i], NULL);
+		if (pixbuf != NULL) {
+			icon_list = g_list_prepend (icon_list, pixbuf);
 		}
 	}
-	
-	gtk_window_set_default_icon_list (icons);
-	
-	g_list_foreach (icons, (GFunc) g_object_unref, NULL);
-	g_list_free (icons);
+
+	gtk_window_set_default_icon_list (icon_list);
+
+	eel_g_list_free_deep_custom (icon_list, (GFunc) g_object_unref, NULL);
 }
 
 static void
@@ -238,10 +189,10 @@ add_sidebar_panel_callback (const char *name,
 			    const char *preference_key,
 			    gpointer callback_data) 
 {
-	g_return_if_fail (name != NULL);
-	g_return_if_fail (iid != NULL);
-	g_return_if_fail (preference_key != NULL);
-	g_return_if_fail (NAUTILUS_IS_WINDOW (callback_data));
+	g_assert (name != NULL);
+	g_assert (iid != NULL);
+	g_assert (preference_key != NULL);
+	g_assert (NAUTILUS_IS_WINDOW (callback_data));
 	
 	eel_preferences_add_callback_while_alive (preference_key,
 						  sidebar_panels_changed_callback,
@@ -250,7 +201,7 @@ add_sidebar_panel_callback (const char *name,
 }
 
 static void
-nautilus_window_init (NautilusWindow *window)
+nautilus_window_instance_init (NautilusWindow *window)
 {
 	window->details = g_new0 (NautilusWindowDetails, 1);
 
@@ -765,7 +716,7 @@ nautilus_window_constructed (NautilusWindow *window)
 					BONOBO_OBJREF (location_bar_wrapper),
 					NULL);
 	bonobo_ui_component_thaw (window->details->shell_ui, NULL);
-	bonobo_object_unref (BONOBO_OBJECT (location_bar_wrapper));
+	bonobo_object_unref (location_bar_wrapper);
 
 	/* initalize the menus and toolbars */
 	nautilus_window_initialize_menus_part_1 (window);
@@ -904,78 +855,85 @@ nautilus_window_unrealize (GtkWidget *widget)
 		window->details->throbber = CORBA_OBJECT_NIL;
 	}
 
-	EEL_CALL_PARENT (GTK_WIDGET_CLASS, unrealize, (widget));
+	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 }
 
-static void 
+static void
 nautilus_window_destroy (GtkObject *object)
 {
 	NautilusWindow *window;
 	
 	window = NAUTILUS_WINDOW (object);
 
-	if (window->details != NULL) {
-		/* Handle the part of destroy that's private to the view
-		 * management.
-		 */
-		nautilus_window_manage_views_destroy (window);
+	nautilus_window_manage_views_destroy (window);
 
-		/* Get rid of all callbacks. */
-		nautilus_window_set_viewed_file (window, NULL);
-		nautilus_window_remove_bookmarks_menu_callback (window);
-		nautilus_window_remove_go_menu_callback (window);
-		nautilus_window_toolbar_remove_theme_callback (window);
+	eel_g_object_list_free (window->sidebar_panels);
+	window->sidebar_panels = NULL;
 
-		if (window->details->ui_idle_id != 0) {
-			gtk_idle_remove (window->details->ui_idle_id);
-		}
-
-		/* Get rid of all owned objects. */
-
-		if (window->details->shell_ui != NULL) {
-			bonobo_ui_component_unset_container (window->details->shell_ui, NULL);
-			bonobo_object_unref (BONOBO_OBJECT (window->details->shell_ui));
-		}
-
-		nautilus_file_unref (window->details->viewed_file);
-
-		g_list_free (window->sidebar_panels);
-
-		free_stored_viewers (window);
-	
-		g_free (window->details->location);
-		eel_g_list_free_deep (window->details->selection);
-		eel_g_list_free_deep (window->details->pending_selection);
-
-		nautilus_window_clear_back_list (window);
-		nautilus_window_clear_forward_list (window);
-
-		if (window->current_location_bookmark != NULL) {
-			g_object_unref (window->current_location_bookmark);
-		}
-		if (window->last_location_bookmark != NULL) {
-			g_object_unref (window->last_location_bookmark);
-		}
-	
-		if (window->status_bar_clear_id != 0) {
-			g_source_remove (window->status_bar_clear_id);
-		}
-
-		if (window->details->ui_container != NULL) {
-			bonobo_object_unref (BONOBO_OBJECT (window->details->ui_container));
-		}
-
-		if (window->details->location_change_at_idle_id != 0) {
-			gtk_idle_remove (window->details->location_change_at_idle_id);
-		}
-
-		g_free (window->details->title);
-	
-		g_free (window->details);
-		window->details = NULL;
+	if (window->content_view != NULL) {
+		gtk_object_destroy (GTK_OBJECT (window->content_view));
+		window->content_view = NULL;
 	}
 
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+}
+
+static void
+nautilus_window_finalize (GObject *object)
+{
+	NautilusWindow *window;
+	
+	window = NAUTILUS_WINDOW (object);
+
+	nautilus_window_manage_views_finalize (window);
+
+	nautilus_window_set_viewed_file (window, NULL);
+	nautilus_window_remove_bookmarks_menu_callback (window);
+	nautilus_window_remove_go_menu_callback (window);
+	nautilus_window_toolbar_remove_theme_callback (window);
+
+	if (window->details->ui_idle_id != 0) {
+		gtk_idle_remove (window->details->ui_idle_id);
+	}
+
+	if (window->details->shell_ui != NULL) {
+		bonobo_ui_component_unset_container (window->details->shell_ui, NULL);
+		bonobo_object_unref (window->details->shell_ui);
+	}
+
+	nautilus_file_unref (window->details->viewed_file);
+
+	free_stored_viewers (window);
+
+	g_free (window->details->location);
+	eel_g_list_free_deep (window->details->selection);
+	eel_g_list_free_deep (window->details->pending_selection);
+
+	nautilus_window_clear_back_list (window);
+	nautilus_window_clear_forward_list (window);
+
+	if (window->current_location_bookmark != NULL) {
+		g_object_unref (window->current_location_bookmark);
+	}
+	if (window->last_location_bookmark != NULL) {
+		g_object_unref (window->last_location_bookmark);
+	}
+
+	if (window->status_bar_clear_id != 0) {
+		g_source_remove (window->status_bar_clear_id);
+	}
+
+	bonobo_object_unref (window->details->ui_container);
+
+	if (window->details->location_change_at_idle_id != 0) {
+		gtk_idle_remove (window->details->location_change_at_idle_id);
+	}
+
+	g_free (window->details->title);
+	
+	g_free (window->details);
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -983,8 +941,8 @@ nautilus_window_save_geometry (NautilusWindow *window)
 {
 	char *geometry_string;
 
-        g_return_if_fail (NAUTILUS_IS_WINDOW (window));
-	g_return_if_fail (GTK_WIDGET_VISIBLE (window));
+        g_assert (NAUTILUS_IS_WINDOW (window));
+	g_assert (GTK_WIDGET_VISIBLE (window));
 
         geometry_string = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
 
@@ -1034,9 +992,10 @@ static void
 nautilus_window_realize (GtkWidget *widget)
 {
         /* Create our GdkWindow */
-	EEL_CALL_PARENT (GTK_WIDGET_CLASS, realize, (widget));
+	GTK_WIDGET_CLASS (parent_class)->realize (widget);
 
         /* Set the mini icon */
+	load_mini_icon ();
 	if (mini_icon_pixmap != NULL) {
 		eel_set_mini_icon (widget->window, mini_icon_pixmap, mini_icon_mask);
 	}
@@ -1052,10 +1011,10 @@ nautilus_window_size_request (GtkWidget		*widget,
 	guint max_width;
 	guint max_height;
 
-	g_return_if_fail (NAUTILUS_IS_WINDOW (widget));
-	g_return_if_fail (requisition != NULL);
+	g_assert (NAUTILUS_IS_WINDOW (widget));
+	g_assert (requisition != NULL);
 
-	EEL_CALL_PARENT (GTK_WIDGET_CLASS, size_request, (widget, requisition));
+	GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
 
 	/* Limit the requisition to be within 90% of the available screen 
 	 * real state.
@@ -1146,8 +1105,8 @@ view_as_menu_switch_views_callback (GtkWidget *widget, gpointer data)
         NautilusWindow *window;
         int viewer_index;
         
-        g_return_if_fail (GTK_IS_MENU_ITEM (widget));
-        g_return_if_fail (NAUTILUS_IS_WINDOW (data));
+        g_assert (GTK_IS_MENU_ITEM (widget));
+        g_assert (NAUTILUS_IS_WINDOW (data));
 
         window = NAUTILUS_WINDOW (data);
 
@@ -1264,7 +1223,7 @@ update_extra_viewer_in_view_as_menus (NautilusWindow *window,
 	/* Remove menu before changing contents so it is resized properly
 	 * when reattached later in this function.
 	 */
-	gtk_widget_ref (menu);
+	g_object_ref (menu);
 	gtk_option_menu_remove_menu (GTK_OPTION_MENU (window->view_as_option_menu));
 
 	/* Remove old menu item, and either remove or add separator. */
@@ -1287,7 +1246,7 @@ update_extra_viewer_in_view_as_menus (NautilusWindow *window,
 	}
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (window->view_as_option_menu), menu);
-	gtk_widget_unref (menu);
+	g_object_unref (menu);
 
 	/* Also update the Bonobo View menu item */
 	if (id == NULL) {
@@ -1418,8 +1377,8 @@ view_as_menu_choose_view_callback (GtkWidget *widget, gpointer data)
 {
         NautilusWindow *window;
         
-        g_return_if_fail (GTK_IS_MENU_ITEM (widget));
-        g_return_if_fail (NAUTILUS_IS_WINDOW (data));
+        g_assert (GTK_IS_MENU_ITEM (widget));
+        g_assert (NAUTILUS_IS_WINDOW (data));
         
         window = NAUTILUS_WINDOW (data);
 
@@ -1466,7 +1425,7 @@ load_view_as_menus_callback (NautilusFile *file,
 
 	window = NAUTILUS_WINDOW (callback_data);
 
-        g_return_if_fail (GTK_IS_OPTION_MENU (window->view_as_option_menu));
+        g_assert (GTK_IS_OPTION_MENU (window->view_as_option_menu));
 
         /* Clear out the menu items created last time. For the option menu, we need do
          * nothing since we replace the entire menu. For the View menu, we have
@@ -1561,7 +1520,7 @@ nautilus_window_add_sidebar_panel (NautilusWindow *window,
 	
 	nautilus_sidebar_add_panel (window->sidebar, sidebar_panel);
 
-	gtk_widget_ref (GTK_WIDGET (sidebar_panel));
+	g_object_ref (sidebar_panel);
 	window->sidebar_panels = g_list_prepend (window->sidebar_panels, sidebar_panel);
 }
 
@@ -1577,7 +1536,7 @@ nautilus_window_remove_sidebar_panel (NautilusWindow *window, NautilusViewFrame
 	
 	nautilus_sidebar_remove_panel (window->sidebar, sidebar_panel);
 	window->sidebar_panels = g_list_remove (window->sidebar_panels, sidebar_panel);
-	gtk_widget_unref (GTK_WIDGET (sidebar_panel));
+	g_object_unref (sidebar_panel);
 }
 
 void
@@ -1645,20 +1604,6 @@ nautilus_window_get_search_mode (NautilusWindow *window)
 	== NAUTILUS_SWITCHABLE_NAVIGATION_BAR_MODE_SEARCH;
 }
 
-void
-nautilus_window_go_web_search (NautilusWindow *window)
-{
-	char *search_web_uri;
-
-	nautilus_window_set_search_mode (window, FALSE);
-
-	search_web_uri = eel_preferences_get (NAUTILUS_PREFERENCES_SEARCH_WEB_URI);
-	g_assert (search_web_uri != NULL);
-	
-	nautilus_window_go_to (window, search_web_uri);
-	g_free (search_web_uri);
-}
-
 void
 nautilus_window_go_home (NautilusWindow *window)
 {
@@ -1772,7 +1717,7 @@ nautilus_send_history_list_changed (void)
 static void
 free_history_list (void)
 {
-	eel_gtk_object_list_free (history_list);
+	eel_g_object_list_free (history_list);
 	history_list = NULL;
 }
 
@@ -1812,7 +1757,7 @@ add_to_history_list (NautilusBookmark *bookmark)
 	GList *l, *next;
 	static gboolean free_history_list_is_set_up;
 
-	g_return_if_fail (NAUTILUS_IS_BOOKMARK (bookmark));
+	g_assert (NAUTILUS_IS_BOOKMARK (bookmark));
 
 	if (!free_history_list_is_set_up) {
 		eel_debug_call_at_shutdown (free_history_list);
@@ -1866,20 +1811,20 @@ nautilus_window_add_current_location_to_history_list (NautilusWindow *window)
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
 	EEL_CALL_METHOD (NAUTILUS_WINDOW_CLASS, window,
-			      add_current_location_to_history_list, (window));
+			 add_current_location_to_history_list, (window));
 }
 
 void
 nautilus_window_clear_forward_list (NautilusWindow *window)
 {
-	eel_gtk_object_list_free (window->forward_list);
+	eel_g_object_list_free (window->forward_list);
 	window->forward_list = NULL;
 }
 
 void
 nautilus_window_clear_back_list (NautilusWindow *window)
 {
-	eel_gtk_object_list_free (window->back_list);
+	eel_g_object_list_free (window->back_list);
 	window->back_list = NULL;
 }
 
@@ -2108,7 +2053,7 @@ nautilus_window_hide_sidebar (NautilusWindow *window)
 	if (window->sidebar == NULL) {
 		return;
 	}
-	nautilus_horizontal_splitter_collapse
+	nautilus_horizontal_splitter_hide
 		(NAUTILUS_HORIZONTAL_SPLITTER (window->content_hbox));
 	nautilus_window_update_show_hide_menu_items (window);
 }
@@ -2119,7 +2064,7 @@ nautilus_window_show_sidebar (NautilusWindow *window)
 	if (window->sidebar == NULL) {
 		return;
 	}
-	nautilus_horizontal_splitter_expand
+	nautilus_horizontal_splitter_show
 		(NAUTILUS_HORIZONTAL_SPLITTER (window->content_hbox));
 	nautilus_window_update_show_hide_menu_items (window);
 }
@@ -2129,8 +2074,8 @@ nautilus_window_sidebar_showing (NautilusWindow *window)
 {
 	g_return_val_if_fail (NAUTILUS_IS_WINDOW (window), FALSE);
 
-	return GTK_IS_PANED (window->content_hbox)
-		&& gtk_paned_get_position (GTK_PANED (window->content_hbox)) != 0;
+	return GTK_IS_PANED (window->content_hbox) 
+		&& nautilus_horizontal_splitter_is_hidden (NAUTILUS_HORIZONTAL_SPLITTER (window->content_hbox));
 }
 
 void 
@@ -2196,7 +2141,7 @@ nautilus_window_show (GtkWidget *widget)
 
 	window = NAUTILUS_WINDOW (widget);
 
-	EEL_CALL_PARENT (GTK_WIDGET_CLASS, show, (widget));
+	GTK_WIDGET_CLASS (parent_class)->show (widget);
 	
 	/* Initially show or hide views based on preferences; once the window is displayed
 	 * these can be controlled on a per-window basis from View menu items. 
@@ -2265,3 +2210,40 @@ nautilus_window_set_viewed_file (NautilusWindow *window,
 	nautilus_file_unref (window->details->viewed_file);
 	window->details->viewed_file = file;
 }
+
+static void
+nautilus_window_class_init (NautilusWindowClass *class)
+{
+	G_OBJECT_CLASS (class)->finalize = nautilus_window_finalize;
+	G_OBJECT_CLASS (class)->get_property = nautilus_window_get_property;
+	G_OBJECT_CLASS (class)->set_property = nautilus_window_set_property;
+	GTK_OBJECT_CLASS (class)->destroy = nautilus_window_destroy;
+	GTK_WIDGET_CLASS (class)->show = nautilus_window_show;
+	GTK_WIDGET_CLASS (class)->unrealize = nautilus_window_unrealize;
+	GTK_WIDGET_CLASS (class)->realize = nautilus_window_realize;
+	GTK_WIDGET_CLASS (class)->size_request = nautilus_window_size_request;
+	class->add_current_location_to_history_list = real_add_current_location_to_history_list;
+	
+	g_object_class_install_property (G_OBJECT_CLASS (class),
+					 ARG_APP_ID,
+					 g_param_spec_string ("app_id",
+							      _("Application ID"),
+							      _("The application ID of the window."),
+							      NULL,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	g_object_class_install_property (G_OBJECT_CLASS (class),
+					 ARG_APP,
+					 g_param_spec_object ("app",
+							      _("Application"),
+							      _("The NautilusApplication associated with this window."),
+							      NAUTILUS_TYPE_APPLICATION,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	
+	/* Set default for all windows. This probably should be done
+	 * in main or NautilusApplication rather than here in case
+	 * some other window is created before the first
+	 * NautilusWindow. Also, do we really want this icon for
+	 * dialogs?
+	 */
+	set_up_default_icon_list ();
+}
