@@ -30,6 +30,7 @@
 
 
 #include <math.h>
+#include <string.h>
 #include <gnome.h>
 #include <gdk/gdk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -79,7 +80,7 @@ struct NautilusZoomControlDetails {
 	GdkPixbuf *number_strip;
 	PrelightMode prelight_mode;
 
-	GdkFont *label_font;
+	PangoLayout *layout;
 
 	gboolean marking_menu_items;
 };
@@ -161,7 +162,7 @@ nautilus_zoom_control_init (NautilusZoomControl *zoom_control)
 	nautilus_zoom_control_load_images (zoom_control);
 	
 	zoom_width = get_zoom_width (zoom_control);
-	gtk_widget_set_usize (GTK_WIDGET (zoom_control), zoom_width, -1);
+	gtk_widget_set_size_request (GTK_WIDGET (zoom_control), zoom_width, -1);
 
 	/* add a callback for when the theme changes */
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_THEME,
@@ -220,6 +221,7 @@ void draw_number (GtkWidget *widget, GdkRectangle *box)
 	int num_v_offset, num_h_offset;
 	NautilusZoomControl *zoom_control;
 	GdkPixbuf *number_pixbuf;
+	PangoRectangle logical_rect;
 
 	zoom_control = NAUTILUS_ZOOM_CONTROL (widget);
 	number_pixbuf = NULL;
@@ -261,16 +263,30 @@ void draw_number (GtkWidget *widget, GdkRectangle *box)
 
 			x += char_width;
 		}
-	} else {				
-		x = num_h_offset + ((widget->allocation.width - gdk_string_width(zoom_control->details->label_font, buffer)) >> 1);  
-		gdk_draw_string (widget->window, zoom_control->details->label_font, temp_gc, x, y, &buffer[0]);
+	} else {
+		widget = GTK_WIDGET (zoom_control);
+
+		pango_layout_set_text (zoom_control->details->layout, buffer, -1);
+		pango_layout_get_pixel_extents (zoom_control->details->layout, NULL, &logical_rect);
+
+		x = num_h_offset + (widget->allocation.width - logical_rect.width)  / 2;
+
+		gtk_paint_layout (widget->style,
+				  widget->window,
+				  GTK_WIDGET_STATE (widget),
+				  FALSE,
+				  box,
+				  widget,
+				  "zoom_control",
+				  x, y,
+				  zoom_control->details->layout);
 	}
 	
 	if (number_pixbuf != zoom_control->details->number_strip) {
 		g_object_unref (number_pixbuf);
 	}
 	
-	gdk_gc_unref(temp_gc);
+	g_object_unref(temp_gc);
 }
 
 /* utilities to simplify drawing */
@@ -406,11 +422,10 @@ nautilus_zoom_control_unload_images (NautilusZoomControl *zoom_control)
 		zoom_control->details->number_strip = NULL;
 	}
 
-	if (zoom_control->details->label_font != NULL) {
-		gdk_font_unref(zoom_control->details->label_font);
-		zoom_control->details->label_font = NULL;
+	if (zoom_control->details->layout != NULL) {
+		g_object_unref (zoom_control->details->layout);
+		zoom_control->details->layout = NULL;
 	}
-
 }
 
 static GdkPixbuf*
@@ -437,18 +452,36 @@ nautilus_zoom_control_update_offsets (NautilusZoomControl *zoom_control)
 static void
 nautilus_zoom_control_load_images (NautilusZoomControl *zoom_control)
 {
+	PangoContext *context;
+	PangoFontDescription *font_desc;
+	char *font_name;
+
 	nautilus_zoom_control_unload_images (zoom_control);
 
 	zoom_control->details->zoom_body_image = load_themed_image ("zoom_body.png");
 	zoom_control->details->zoom_decrement_image = load_themed_image ("decrement.png");
 	zoom_control->details->zoom_increment_image = load_themed_image ("increment.png");
+
 	zoom_control->details->number_strip = load_themed_image ("number_strip.png");
-	
+
 	if (zoom_control->details->number_strip == NULL) {
-		/* Note to localizers: this font is used for the number in the
-		 * zoom control widget.
-		 */
-		zoom_control->details->label_font = gdk_fontset_load (_("-bitstream-courier-medium-r-normal-*-9-*-*-*-*-*-*-*"));
+		context = gtk_widget_get_pango_context (GTK_WIDGET (zoom_control));
+		zoom_control->details->layout = pango_layout_new (context);
+		
+		font_name = nautilus_theme_get_theme_data ("zoom", "font_name");
+		if (!font_name) {
+			font_name = g_strdup ("Mono 9");
+		}
+
+		font_desc = pango_font_description_from_string (font_name);
+
+		if (font_desc) {
+			pango_layout_set_font_description (zoom_control->details->layout,
+							   font_desc);
+		}
+
+		pango_font_description_free (font_desc);
+		g_free (font_name);
 	}
 
 	nautilus_zoom_control_update_offsets (zoom_control);
@@ -513,13 +546,12 @@ create_zoom_menu_item (GtkMenu *menu, GtkWidget *widget, float zoom_level,
 
 	radio_item_group = previous_radio_item == NULL
 		? NULL
-		: gtk_radio_menu_item_group (previous_radio_item);
+		: gtk_radio_menu_item_get_group (previous_radio_item);
 	menu_item = gtk_radio_menu_item_new_with_label (radio_item_group, item_text);
 
 	zoom_level_ptr = g_new (double, 1);
 	*zoom_level_ptr = zoom_level;
 
-	gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (menu_item), TRUE);
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), 
 					zoom_level == zoom_control->details->zoom_level);
 	
@@ -529,7 +561,7 @@ create_zoom_menu_item (GtkMenu *menu, GtkWidget *widget, float zoom_level,
 			    zoom_control);
 
   	gtk_widget_show (menu_item);
-	gtk_menu_append (menu, menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 
 	zoom_control->details->marking_menu_items = FALSE;
 
@@ -746,7 +778,7 @@ nautilus_zoom_control_class_init (NautilusZoomControlClass *zoom_control_class)
 		              G_STRUCT_OFFSET (NautilusZoomControlClass, 
 						   zoom_in),
 		              NULL, NULL,
-		              gtk_marshal_VOID__VOID,
+		              g_cclosure_marshal_VOID__VOID,
 		              G_TYPE_NONE, 0);
 
 	signals[ZOOM_OUT] =
@@ -756,7 +788,7 @@ nautilus_zoom_control_class_init (NautilusZoomControlClass *zoom_control_class)
 		              G_STRUCT_OFFSET (NautilusZoomControlClass, 
 						   zoom_out),
 		              NULL, NULL,
-		              gtk_marshal_VOID__VOID,
+		              g_cclosure_marshal_VOID__VOID,
 		              G_TYPE_NONE, 0);
 
 	signals[ZOOM_TO_LEVEL] =
@@ -778,6 +810,6 @@ nautilus_zoom_control_class_init (NautilusZoomControlClass *zoom_control_class)
 		              G_STRUCT_OFFSET (NautilusZoomControlClass, 
 						   zoom_to_fit),
 		              NULL, NULL,
-		              gtk_marshal_VOID__VOID,
+		              g_cclosure_marshal_VOID__VOID,
 		              G_TYPE_NONE, 0);
 }
